package com.cht.tas.ap;

import jakarta.annotation.PreDestroy;
import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.nio.charset.StandardCharsets;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * 處理 callout API 撥出給用戶時，用戶電話互動產生的 MQTT 訊息
 * 處理用戶回撥 (call-in) 到 TAS 電話服務時，用戶電話互動所產生的 MQTT 訊息
 */
@Component
public class IVRMqtt {
    private final Logger logger = LoggerFactory.getLogger(IVRMqtt.class);

    @Value("${mqtt.phone-conn.calloutResultTopic}")
    private String calloutResultTopic = "";

    @Value("${mqtt.phone-conn.eventTopic}")
    private String eventTopic = "";

    @Value("${mqtt.phone-conn.actionDebugTopic}")
    private String actionDebugTopic = "";

    @Autowired
    private IoTTas ioTTas;

    @Autowired
    private CalloutEvent calloutEvent;

    @Autowired
    private IVREvent ivrEvent;

    //建立一個專門用於 publish MQTT 訊息的 thread pool
    private final ExecutorService publishService = Executors.newSingleThreadExecutor();

    public static int serialNumber = 0;

    public void setSubscribeAllTopic() {
        setSubscribeActionDebugTopic();
        setSubscribeEventTopic();
        setSubscribeCalloutResultTopic();
    }

    //訂閱 MQTT calloutResult topic 訊息
    public void setSubscribeCalloutResultTopic() {
        MqttTas.setSubscribeTopicResp(calloutResultTopic, (topic, message) -> {
            //印出從 calloutResult topic 收到的訊息內容
            String msg = new String(message.getPayload(), StandardCharsets.UTF_8);
            JSONObject reqJObjs = new JSONObject(msg);
            logger.debug("MQTT RECV:{}", reqJObjs);
        });

    }

    //若送入到 callAction topic 訊息有錯誤時，系統會透過此 callActionDebug topic 印出除錯訊息
    public void setSubscribeActionDebugTopic() {
        MqttTas.setSubscribeTopicResp(actionDebugTopic, ((topic, message) -> {
            String msg = new String(message.getPayload(), StandardCharsets.UTF_8);
            logger.debug("DEBUG RECV:" + msg);
        }));
    }

    //訂閱 callEvent topic，接收用戶電話接通後的即時互動訊息，以決定如何回應用戶
    public void setSubscribeEventTopic() {
        MqttTas.setSubscribeTopicResp(eventTopic, (topic, message) -> {
            String msg = new String(message.getPayload(), StandardCharsets.UTF_8);
            JSONObject reqJObjs = new JSONObject(msg);
            String type = reqJObjs.getString("type");
            String phone = reqJObjs.getString("phone");

            logger.debug("msg: {}", msg);
            logger.debug("type: {}", type);

            //若是收到的 MQTT 事件，其 type 屬性為 calloutEvent 代表為 callout API 時產生的，若是 type 屬性值為 event 代表為
            //用戶回撥電話 (call-in) 到系統時產生的事件訊息
            if ("calloutEvent".equals(type)) {  //ap event
                String event = reqJObjs.getString("event");
                if (event.equals("CallEstablished")) {
                    //呼叫建立
                    logger.debug("CallEstablished");
                } else if (event.equals("CallHangUp")) {
                    //呼叫結束
                    logger.debug("CallHangUp");
                }
            } else if ("event".equals(type)) {  //call-in event

            }

            //若是從 callEvent 收到的 MQTT 訊息，其 type 屬性為 request 代表為用戶回撥 (call-in) 產生的訊息
            //收到 MQTT Call-in IVR 互動選單的 request
            if ("request".equals(type)) {
                //node 值的內容，若是有用戶輸入的問券回應值時，型式為 node: MyAskNode,1  代表用戶按下數字鍵 1
                //拆解 node 節點的內容及參數
                String[] mArgs = reqJObjs.getString("node").split(",");

                //節點內容
                String node = mArgs[0];

                //節點參數
                String param = "";
                if (mArgs.length > 1) {
                    param = mArgs[1];
                }

                String finalParam = param;
                publishService.submit(() -> {
                    switch (node) {
                        //接通後，進入 MAIN 節點
                        case "MAIN": {
                            String value = ioTTas.phonePools.get(phone);
                            if (value == null) { //查無此號碼，為非本次問券調查的對象
                                ivrEvent.wrongAsk(reqJObjs);
                            } else if (value.equals("0")) { //該號碼尚未回答問券
                                ivrEvent.menu(reqJObjs);
                            } else {                        //已回答過本問券
                                logger.debug("already ask");
                                ivrEvent.alreadyAsk(reqJObjs);
                            }

                            break;
                        }
                        //節點內容
                        case "MyAskNode": {
                            ivrEvent.myAskNode(reqJObjs, finalParam);
                            break;
                        }
                        default: {
                            logger.debug("default:{}", node);
                        }
                    }
                });
            }

            //若是從 callEvent 收到的 MQTT 訊息，其 type 屬性為 calloutRequest 代表為使用 callout API 主動 callout 給用戶時產生的訊息
            //收到 MQTT Callout 互動選單的 request
            if ("calloutRequest".equals(type)) {

                //node 值的內容，若是有用戶輸入的問券回應值時，型式為 node: MyAskNode,1  代表用戶按下數字鍵 1
                //拆解 node 節點的內容及參數
                String[] mArgs = reqJObjs.getString("node").split(",");

                //節點內容
                String node = mArgs[0];

                //節點參數
                String param = "";
                if (mArgs.length > 1) {
                    param = mArgs[1];
                }

                String finalParam = param;
                publishService.submit(() -> {
                    switch (node) {
                        //接通後，進入 MAIN 節點
                        case "MAIN": {
                            calloutEvent.menu(reqJObjs);
                            break;
                        }
                        //節點內容
                        case "MyAskNode": {
                            calloutEvent.myAskNode(reqJObjs, finalParam);
                            break;
                        }
                        default: {
                            logger.debug("default:{}", node);
                        }
                    }
                });
            }
        });
    }

    @PreDestroy
    private void destory() {
        publishService.shutdown();
    }
}
