package com.cht.tas.ap;

import jakarta.annotation.PostConstruct;
import jakarta.annotation.PreDestroy;
import org.eclipse.paho.client.mqttv3.*;
import org.eclipse.paho.client.mqttv3.persist.MemoryPersistence;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.security.SecureRandom;
import java.util.Random;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

/**
 * 用於處理與伺服器端 MQTT 連線的程式，也執行 MQTT topic 的 subscribe 與 publish 操作
 */
@Component
public class MqttTas {
    private static final Logger logger = LoggerFactory.getLogger(MqttTas.class);

    @Value("${mqtt.phone-conn.brokerUrl}")
    private String brokerUrl = "";

    @Value("${mqtt.phone-conn.apiKey}")
    private String apiKey = "";

    @Autowired
    private IVRMqtt IVRMqtt;

    private static MqttClient mClient;
    private final Random random = new SecureRandom();
    //建立用於與伺服器斷 MQTT 連線中斷時，用於自動重新連線的 thread pool
    private final ScheduledExecutorService reconnectMQTTThread = Executors.newSingleThreadScheduledExecutor();

    @PostConstruct
    public void init() {
        try {
            logger.debug("MQTT TAS start");
            String clientId = MqttClient.generateClientId();

            mClient = new MqttClient(brokerUrl, clientId, new MemoryPersistence());
            mClient.setCallback(new MqttCallback() {

                @Override
                public void connectionLost(Throwable cause) {
                    logger.debug("Connection to MQTT broker lost! : ", cause);
                    int retryDelayTime = 1 + random.nextInt(10);
                    logger.debug("schedule retry connect to Mqtt Iot Server in {} s", retryDelayTime);
                    reconnectMQTTThread.schedule(() -> connectMQTTPhoneServer(), retryDelayTime, TimeUnit.SECONDS);
                }

                @Override
                public void messageArrived(String topic, MqttMessage message) {
                    logger.debug("recv topic:{}", topic);

                }

                @Override
                public void deliveryComplete(IMqttDeliveryToken token) {
                    //Log.debug("deliveryComplete() is called");
                }
            });

            connectMQTTPhoneServer();

        } catch (MqttException ex) {
            logger.debug("MqttException:", ex);
        }
    }


    private void connectMQTTPhoneServer() {
        MqttConnectOptions connOpts = new MqttConnectOptions();
        connOpts.setPassword(apiKey.toCharArray());
        connOpts.setUserName(apiKey);
        connOpts.setCleanSession(true);
        try {
            mClient.connect(connOpts);
            IVRMqtt.setSubscribeAllTopic();
            logger.debug("Mqtt Phone Connected");
        } catch (MqttException e) {
            logger.error("MQTT Exception message: {}, reason code: {}", e.getMessage(), e.getReasonCode());
            int retryDelayTime = 30 + random.nextInt(10);
            logger.debug("schedule retry connect to Mqtt Iot Server in {} s", retryDelayTime);
            reconnectMQTTThread.schedule(() -> connectMQTTPhoneServer(), retryDelayTime, TimeUnit.SECONDS);
        }
    }

    @PreDestroy
    public void destroy() {
        if (mClient != null) {
            try {
                logger.debug("destroy");
                mClient.disconnect();
            } catch (MqttException e) {
                logger.error("destroy() got exception:", e);
            }
        }
    }

    static void setSubscribeTopicResp(String topic, IMqttMessageListener messageListener) {
        try {
            mClient.subscribe(topic, messageListener);
        } catch (MqttException e) {
            logger.error("setSubscribeTopicResp() got exception:", e);
        }
    }

    static void publish(String topic, MqttMessage message) {
        try {
            mClient.publish(topic, message);
        } catch (MqttException e) {
            logger.error("publish() got exception:", e);
        }

    }

}
